/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     Main

   PURPOSE
     driver to run the planner
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY

***************************************************************************/


#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <time.h>
#include "planner1.H"
#include "stat.H"
#include "ctrl.H"
#include "utilities.H"


/*---------------------- Global Variables -----------------------------*/

//   global variable, visible to all objects

/* Ct
 *   control the global configuration and behavior of the planner
 */
Controller Ct;

/* G
 *   contain two major components of the planner: C-space,  and workspace
 *   workspace is in turn composed of robot and obstacles
 */
Planner1 G;

/* Stat
 *   gather statistics of the program
 */
Statistics Stats;

/*-------------------------  Functions ---------------------------------*/

/* printReport
 *   print a report of execution result
 */

void printReport(ostream& os)
{
    time_t currTime = time(NULL);
    char *currTime_s = ctime(&currTime);
    
    os << "\n\n\n";
    os <<"----------------------  RRS Execution Report --------------------\n";
    os <<"                      " <<  currTime_s << "\n";
    os << Ct << Stats << G;
}   


/* handler
 *   	
 */

void handler(int sig)
{
  switch (sig){
  case SIGINT:
    times( & Stats.qEnd );
    printReport(cout);
    break;
  }
  exit(0);
}

/* cleanUp
 *    deallocate all objects on heap
 */

void cleanUp()
{       
    
}   

void usage()
{
  cerr << "Usage: \n\nrss -r filename -c filename -e filename -cs filname "
       << " [-s] [-i] [-u] [-t]" << endl;
  cerr << "\t-r filename \t specify data file for the robot\n";
  cerr << "\t-e filename \t specify data file for obstacles\n";
  cerr << "\t-cs filename \t specify data file for c-space\n";
  cerr << "\t-c filename \t data file configuring the planner\n";
  cerr << "\t-u \t\t expand the known c-space from initial configuration only\n";
  cerr << "\t-s \t\t smooth the output path\n";
  cerr << "\t-i \t\t interpolate the final path\n";
  cerr << "\t-t \t\t dump out a trace of the execution\n";
  exit(0);                            
}       


int main(int argc, char *argv[])
{
    int rflag=0, eflag=0, cflag=0, sflag=0;
    ifstream rsrc, esrc, asrc;

    // configuration file must be loaded before the robot and obstacle
    // data files
    for (int i=1; i < argc; i++) {

	// data file for the robot
	if ( ! strcmp( argv[i], "-r") ){
	    rsrc.open( argv[++i] );
	    sprintf( Stats.robotName, "%s", argv[i] );
	    if (!rsrc) 
		error("rss: could not open file ", argv[i] );
	    rflag = 1;
	}

	// data file for the obstacles
	else if ( ! strcmp( argv[i], "-e") ) {
	    esrc.open( argv[++i] );
	    sprintf( Stats.obsName, "%s", argv[i] );
	    if (!esrc)
		error("rss: could not open file ", argv[i] );
	    eflag = 1;
	}

	// data file for the c-space
	else if ( ! strcmp( argv[i], "-cs") ) {
	    asrc.open(argv[++i]);
	    if (!asrc) 
		error("rss: could not open file ", argv[i]);
	    sflag = 1;
	}
			 
	// data configuring the planner 
	else if ( ! strcmp( argv[i], "-c") ) {
	    ifstream src( argv[++i] );
	    char tmpFileName[32];

	    if (src) {
		sprintf(tmpFileName, "%s%d", "/usr/tmp/rss.", getpid() );
		fstream target;
		target.open( tmpFileName, ios::out );
		stripComment( src, target ); 
		target.close();
		target.open(tmpFileName, ios::in);
		target >> Ct;
		remove(tmpFileName);	/* remove the temporary file */
	    }
	    else 
		error("rss: could not open file ", argv[i] );
	    cflag = 1;
	}

	// expand only from the init config
	else if ( ! strcmp(argv[i], "-u")) {
	    Ct.uni = 1;
	}
	     
	//  smooth the path
	else if (! strcmp(argv[i], "-s") ) {
	    Ct.sm = 1;
	}

	// interpolate the final path
	else if (! strcmp(argv[i], "-i") ) {
	    Ct.it = 1;
	}

	// dump out trace of the execution
	else if (! strcmp(argv[i], "-t") ) {
	    Ct.tr = 1;
	}

	else 
	    error("rss: unrecognized option", NULL);
    }

    if ( !( rflag && eflag && cflag && sflag) )
	usage(); 

    signal(SIGINT,handler);		/* register handler */

    G.bgn(rsrc, esrc, asrc);		/* initalize the planner */

    cin >> G;				/* read init. and goal config */
    ofstream path("path0.dat");		/* output file */
    Stats.reset();			/* reset stats */
    G.query(path);			/* query  */
    printReport(cout);

    cleanUp();
	
    return 0;
}
