/* Copyright (c) 1996 by Stanford University.  All Rights Reserved */

/***************************************************************************
   NAME
     Model
   PURPOSE
     data structures for storing models
     
   NOTES
     
   AUTHOR
     David Hsu  
   
     dyhsu@cs.stanford.edu
     http://robotics.stanford.edu/~dyhsu 
                  
   HISTORY
     created July, 1996

***************************************************************************/


#include <iostream.h>
#include "math++.h"
#include "dist.h"
#include "utilities.H"
#include "model.H"
#include "global.H"

/*----------------------- class Model -----------------------------*/

ostream& operator<< (ostream& os, Model& m)
{
    m.print(os);
    return os;
}


/*----------------------- class Phdn -----------------------------*/

void Phdn::end()
{
    calcRadius();
    buildModel();
}

void Phdn::calcRadius()
{	
    Coord max = 0.0, d;

    for (int i=0; i < vcount; i++) {
	d = sqrt_pp( v[i][X] * v[i][X] + v[i][Y] * v[i][Y] + v[i][Z]*v[i][Z]);
	max = MAX( max, d);
    }
    maxRadius = max;
}

void Phdn::buildModel()
{
    Point3 tri[3];

    bgnsobj3();
    for (int i=0; i < tcount; i++) {
	// copy the coords of triangle vertices into tri
	for (int j=0; j < 3; j++) { 
	    tri[j].x = v[t[i][j]][X];
	    tri[j].y = v[t[i][j]][Y];
	    tri[j].z = v[t[i][j]][Z];  
	}
	spoly3( tri, 3, tol, Ct.rmax, NULL);
    }
    handle = endsobj3();
}



void  Phdn::print(ostream& os)
{
    int i;

    os << char(type) << ' ';
    os << vcount << ' ' << tcount << ' ' << tol << endl;
    for (i=0; i < vcount; i++)		/* write vertices */
	os << v[i][X] << ' ' << v[i][Y] << ' ' << v[i][Z] << endl;
    os << endl;
    for (i=0; i < tcount; i++)		/* write faces    */
	os << t[i][0] << ' ' <<  t[i][1] << ' ' << t[i][2] << endl;
    os << endl;
}


istream& operator>> (istream& is, Phdn& p)
{
    int nv, nt, i;

    is >> nv >> nt >> p.tol;
    p.bgn(nv, nt);

    for (i=0; i < nv; i++)		/* read vertices */
	is >> (p.v[i][X]) >> p.v[i][Y] >> p.v[i][Z];
    for (i=0; i < nt; i++)		/* read faces    */
	is >> p.t[i][0] >> p.t[i][1] >> p.t[i][2]  ;

    p.end();
    return is;
}

ostream& operator<< (ostream& os, Phdn & p)
{
    p.print(os);
    return os;
}


/*----------------------- class Moray -----------------------------*/

Moray::~Moray()
{
    for (int i=0; i<mcount; i++)	/* delete each model first */
	if (m[i]) delete m[i];
    delete [] m;
}


void Moray::calcRadius()
{
    Coord max=0.0;

    for (int i=0; i<mcount; i++) {
	m[i]->calcRadius();
	max = MAX( max, m[i]->radius() );
    }
    maxRadius = max;
}

void Moray::buildModel()
{
    int i;

    // build data structure for each model in moray
    for (i=0; i < mcount; i++) 
	m[i]->buildModel();

    // group them together if there is more than one model
    if ( mcount > 1 ) {
	bgnsobj3();
	for (i=0; i < mcount; i++)
	    ssobj3( m[i]->model(), IFRAME, NULL);
	handle = endsobj3();
    }
    else
	handle = m[0]->model();
}

void Moray::end()
{
    calcRadius();
    buildModel();
}

istream& operator>> (istream& is, Moray& m)
{
    int i, mc;
    Phdn *pmodel;
    char type;

    is >> mc;				/* number of models */
    m.bgn(mc);

    for (i=0; i < mc; i++ ) {		/* read in the models */
	is >> type;
	switch (type) {
	case P_TYPE:			/* read polyhedral model */
	    pmodel = new Phdn;
	    is >> (*pmodel);
	    break;
	default:
	    error("Moray: unrecognized model type");
	}
	m.model(i, pmodel);		/* add the model */
    }

    m.end();
    return is;
}

ostream& operator<< (ostream& os, Moray& mry)
{
    os << mry.mcount << endl;
    for (int i=0; i < mry.mcount; i++)
	os <<  *(mry.m[i]) ;
    os << "max radius = " << mry.maxRadius << endl;
    return os;
}
